#pragma once

#include <windows.h>
#include "netana_user.h"
#include "netana_errors.h"
#include <time.h>

typedef int32_t(APIENTRY *PFN_netana_driver_information   )(uint32_t ulDrvInfoSize, NETANA_DRIVER_INFORMATION_T* ptDrvInfo);
typedef int32_t(APIENTRY *PFN_netana_enum_device          )(uint32_t ulCardNr, uint32_t ulDevInfoSize, NETANA_DEVICE_INFORMATION_T* ptDevInfo);
typedef int32_t(APIENTRY *PFN_netana_open_device          )(char* szDevice, NETANA_HANDLE* phDev);
typedef int32_t(APIENTRY *PFN_netana_start_capture        )(NETANA_HANDLE hDev,             uint32_t ulCaptureMode, 
                                                            uint32_t ulPortEnableMask,      uint32_t ulMacMode, 
                                                            uint64_t ullReferenceTime,      PFN_STATUS_CALLBACK pfnStatus,  
                                                            PFN_DATA_CALLBACK pfnData,      void* pvUser);
typedef int32_t(APIENTRY *PFN_netana_stop_capture         )(NETANA_HANDLE hDev);
typedef int32_t(APIENTRY *PFN_netana_close_device         )(NETANA_HANDLE hDev);
typedef int32_t(APIENTRY *PFN_netana_get_error_description)(int32_t iError, uint32_t ulBufferSize, char* szBuffer);
typedef int32_t(APIENTRY *PFN_netana_device_info          )(NETANA_HANDLE hDev, uint32_t ulDevInfoSize, NETANA_DEVICE_INFORMATION_T* ptDevInfo);
typedef int32_t(APIENTRY *PFN_netana_get_portstat         )(NETANA_HANDLE hDev, uint32_t ulPort, uint32_t ulSize, NETANA_PORT_STATE_T* ptStatus);
typedef int32_t(APIENTRY *PFN_netana_get_state            )(NETANA_HANDLE hDev, uint32_t* pulCaptureState, uint32_t* pulCaptureError);
typedef int32_t(APIENTRY *PFN_netana_access_phy_reg       )(NETANA_HANDLE hDev, uint32_t ulDirection, 
                                                            uint8_t bPhyNum,    uint8_t bPhyReg,  
                                                            uint16_t* pusValue, uint32_t ulTimeout);
typedef int32_t(APIENTRY *PFN_netana_get_gpio_mode        )(NETANA_HANDLE hDev, uint32_t ulGpio, NETANA_GPIO_MODE_T* ptMode);
typedef int32_t(APIENTRY *PFN_netana_set_gpio_mode        )(NETANA_HANDLE hDev, uint32_t ulGpio, NETANA_GPIO_MODE_T* ptMode);
typedef int32_t(APIENTRY *PFN_netana_set_gpio_voltage     )(NETANA_HANDLE hDev, uint32_t ulGpioSelMask, uint32_t ulGpioVoltageSel);
typedef int32_t(APIENTRY *PFN_netana_get_filter           )(NETANA_HANDLE hDev, uint32_t ulPort, 
                                                            NETANA_FILTER_T* ptFilterA, NETANA_FILTER_T* ptFilterB, 
                                                            uint32_t* pulRelationShip);
typedef int32_t(APIENTRY *PFN_netana_set_filter           )(NETANA_HANDLE hDev, uint32_t ulPort, 
                                                            NETANA_FILTER_T* ptFilterA, NETANA_FILTER_T* ptFilterB, 
                                                            uint32_t ulRelationShip);
typedef int32_t(APIENTRY *PFN_netana_set_gpio_output      )(NETANA_HANDLE hDev, uint32_t ulGpio, uint32_t ulLevel);
typedef int32_t(APIENTRY *PFN_netana_resync_time          )(NETANA_HANDLE hDev, uint64_t ullReferenceTime);
typedef int32_t(APIENTRY *PFN_netana_config_pi_controller )(NETANA_HANDLE hDev, uint32_t ulP, uint32_t ulI);

typedef int32_t(APIENTRY *PFN_netana_mngmt_exec_cmd       )( uint32_t ulCommandId,
                                                             void*  ptInputParameter,  uint32_t ulInputParameterSize,
                                                             void*  ptOutputParameter, uint32_t ulOutputParameterSize);

typedef struct NETANADRIVER_FUNCTIONStag
{
  HINSTANCE hDll;
  PFN_netana_driver_information     pfn_netana_driver_information;
  PFN_netana_enum_device            pfn_netana_enum_device;
  PFN_netana_open_device            pfn_netana_open_device; 
  PFN_netana_start_capture          pfn_netana_start_capture; 
  PFN_netana_stop_capture           pfn_netana_stop_capture;     
  PFN_netana_close_device           pfn_netana_close_device;   
  //PFN_netana_get_error_description  pfn_netana_get_error_description;
  PFN_netana_device_info            pfn_netana_device_info;
  //PFN_netana_get_portstat           pfn_netana_get_portstat;
  //PFN_netana_get_state              pfn_netana_get_state;
  PFN_netana_access_phy_reg         pfn_netana_access_phy_reg;
  //PFN_netana_get_gpio_mode          pfn_netana_get_gpio_mode;
  PFN_netana_set_gpio_mode          pfn_netana_set_gpio_mode;
  PFN_netana_set_gpio_voltage       pfn_netana_set_gpio_voltage;
  //PFN_netana_get_filter             pfn_netana_get_filter;
  //PFN_netana_set_filter             pfn_netana_set_filter;
  //PFN_netana_set_gpio_output        pfn_netana_set_gpio_output;
  //PFN_netana_resync_time            pfn_netana_resync_time;
  //PFN_netana_config_pi_controller   pfn_netana_config_pi_controller;
  PFN_netana_mngmt_exec_cmd         pfn_netana_mngmt_exec_cmd;

} NETANADRIVER_FUNCTIONS, *PNETANADRIVER_FUNCTIONS;

typedef struct DEC_NETANA_ETH_FRAME_Ttag
{
	NETANA_FRAME_HEADER_T   tHeader;
	uint8_t									abData[1600];
} DEC_NETANA_ETH_FRAME_T;

typedef struct netAna_ADAPTER_
{
  UINT              received;  // overall packets received
  UINT              dropped;  // overall packets dropped
  UINT              captured;  // overall packets dropped
  DWORD			        mode;
  DWORD			        readTimeout;
  DWORD			        minToCopy;
  DWORD             BufferSize;
  CRITICAL_SECTION  tcsGetOldestLock;
  int64_t		        llLatestWinTime;
  uint64_t		      ullNewestFrameTime;
  uint32_t          curQueSize;
  uint32_t          fOverFlowActive;
  uint32_t          status;
  BOOLEAN           startCapture;
  void*             avRxPorts[8];
  NETANA_HANDLE     netAnaHandle;
  HANDLE			      hReadEvent;
}netAna_ADAPTER;


#ifdef __cplusplus
extern "C" {
#endif


BOOLEAN NetAnaLoadApiDll(NETANADRIVER_FUNCTIONS *pfnFnc);
BOOLEAN NetAnalyzerOpenAdapter(NETANADRIVER_FUNCTIONS NetAnaPfnFnc, netAna_ADAPTER* netAnaAdapter, PCHAR AdapterName);
BOOLEAN NetAnalyzerCloseAdapter(NETANADRIVER_FUNCTIONS NetAnaPfnFnc, netAna_ADAPTER* netAnaAdapter);
BOOLEAN NetAnalyzerReceivePacket(netAna_ADAPTER* netAnaAdapter, PUCHAR Buffer, UINT Length, DWORD *actualLength);
BOOLEAN NetAnalyzerSetMode(netAna_ADAPTER* netAnaAdapter, DWORD mode);
BOOLEAN NetAnalyzerSetReadTimeout(netAna_ADAPTER* netAnaAdapter, DWORD readTimeout);
BOOLEAN NetAnalyzerSetMinToCopy(netAna_ADAPTER* netAnaAdapter, DWORD minToCopy);
BOOLEAN NetAnalyzerSetBufferSize(netAna_ADAPTER* netAnaAdapter, DWORD BufferSize);
HANDLE NetAnalyzerGetReadEvent(netAna_ADAPTER* netAnaAdapter);
CHAR* NetAnalyzerGetName(NETANADRIVER_FUNCTIONS NetAnaPfnFnc, int number);
int NetAnalyzerCountDevices(NETANADRIVER_FUNCTIONS NetAnaPfnFnc);

#ifdef __cplusplus
}
#endif
